@@ -212,6 +212,7 @@ int rc_read_config(char *filename)
 
 		if ((pos = strcspn(p, "\t ")) == 0) {
 			error("%s: line %d: bogus format: %s", filename, line, p);
+			fclose(configfd);
 			return (-1);
 		}
 
@@ -224,6 +225,7 @@ int rc_read_config(char *filename)
 
 		if (option->status != ST_UNDEF) {
 			error("%s: line %d: duplicate option line: %s", filename, line, p);
+			fclose(configfd);
 			return (-1);
 		}
 
@@ -234,18 +236,22 @@ int rc_read_config(char *filename)
 		switch (option->type) {
 			case OT_STR:
 				 if (set_option_str(filename, line, option, p) < 0)
+					fclose(configfd);
 					return (-1);
 				break;
 			case OT_INT:
 				 if (set_option_int(filename, line, option, p) < 0)
+					fclose(configfd);
 					return (-1);
 				break;
 			case OT_SRV:
 				 if (set_option_srv(filename, line, option, p) < 0)
+					fclose(configfd);
 					return (-1);
 				break;
 			case OT_AUO:
 				 if (set_option_auo(filename, line, option, p) < 0)
+					fclose(configfd);
 					return (-1);
 				break;
 			default:
@@ -525,7 +531,6 @@ int rc_find_server (char *server_name, UINT4 *ip_addr, char *secret)
 	if (result == 0)
 	{
 		memset (buffer, '\0', sizeof (buffer));
-		memset (secret, '\0', sizeof (secret));
 		error("rc_find_server: couldn't find RADIUS server %s in %s",
 		      server_name, rc_conf_str("servers"));
 		return (-1);
